@@ -20,6 +20,10 @@ ALGORITHMS = ["RS256"]
 EXPIRATION = 1000
 
 
+class AuthError(Exception):
+    status_code = 500
+
+
 def hash_digest(body):
     myhash = hashlib.sha512()
     myhash.update(body)
@@ -32,16 +36,16 @@ def get_token_header(auth_type, request):
     """
     auth = request.headers.get("Authorization", None)
     if not auth:
-        raise Exception("No Authorization Header")
+        raise AuthError("No Authorization Header")
 
     auth_parts = auth.split()
 
     if auth_parts[0].lower() != auth_type.lower():
-        raise Exception("Authorization Header must start with {}".format(auth_type))
+        raise AuthError("Authorization Header must start with {}".format(auth_type))
     elif len(auth_parts) == 1:
-        raise Exception("No Signature attached")
+        raise AuthError("No Signature attached")
     elif len(auth_parts) > 2:
-        raise Exception("Invalid Header")
+        raise AuthError("Invalid Header")
     token = auth_parts[1]
 
     return token
@@ -74,30 +78,34 @@ class Secure:
         self.sign_makers = {}
         self.sign_checkers = {}
         if len(auth_makers) > 0:
-            for a in auth_makers:
-                if a["type"] == "auth0":
-                    self.auth_makers[a["name"]] = Authenticator(**a["config"])
+            for k, v in auth_makers.items():
+                if v["type"] == "auth0":
+                    config = v["config"]
+                    config["domain"] = "https://{}/oauth/token".format(config["domain"])
+                    self.auth_makers[k] = Authenticator(**config)
+                else:
+                    self.auth_makers[k] = Authenticator(**v["config"])
         if len(auth_checkers) > 0:
-            for a in auth_checkers:
-                self.auth_checkers[a["name"]] = Authorizer(
-                    audience=a["audience"], keySet=make_keySet(keySet, a["issuers"])
+            for k, v in auth_checkers.items():
+                self.auth_checkers[k] = Authorizer(
+                    audience=v["audience"], keySet=make_keySet(keySet, v["issuers"])
                 )
         if len(sign_makers) > 0:
-            for s in sign_makers:
-                self.sign_makers[s["name"]] = Signer(
-                    issuer=s["issuer"], private_key=s["private_key"]
+            for k, v in sign_makers.items():
+                self.sign_makers[k] = Signer(
+                    issuer=v["issuer"], private_key=v["private_key"]
                 )
         if len(sign_checkers) > 0:
-            for s in sign_checkers:
-                self.sign_checkers[s["name"]] = Signature_Verifyer(
-                    audience=s["audience"], keySet=make_keySet(keySet, s["issuers"])
+            for k, v in sign_checkers.items():
+                self.sign_checkers[k] = Signature_Verifyer(
+                    audience=v["audience"], keySet=make_keySet(keySet, v["issuers"])
                 )
 
     def validate_scope(self, scope, token=None):
-        if not token:
-            token_payload = g.authorize_token_payload
-        else:
+        try:
             token_payload = jwt.get_unverified_claims(token)
+        except:
+            raise AuthError("Error in token access")
         if scope == None or scope == "":
             return True
         if token_payload.get("scope"):
@@ -117,14 +125,13 @@ class Secure:
                     elif len(self.auth_checkers) > 1:
                         auth_checker = self.auth_checkers[name]
                     else:
-                        raise Exception("No Auth Checker defined in Config")
+                        raise AuthError("No Auth Checker defined in Config")
                     token = get_token_header("Bearer", request)
                     payload = auth_checker.decode(token, issuers)
-                    # g.authorize_token_payload = payload
                 except:
-                    raise Exception("Invalid Header", 401)
+                    raise AuthError("Invalid Bearer Token")
                 if not self.validate_scope(scope, token):
-                    raise Exception("Invalid Scope", 401)
+                    raise AuthError("Invalid Scope")
                 return f(*args, **kwargs)
 
             return decorated
@@ -141,7 +148,7 @@ class Secure:
         elif len(self.auth_makers) > 1:
             auth_maker = self.auth_makers[name]
         else:
-            raise Exception("No Auth Maker defined in Config")
+            raise AuthError("No Auth Maker defined in Config")
         return auth_maker.access_token
 
     def sign_check(self, name=None, json_dec=False, issuers=None):
@@ -153,12 +160,12 @@ class Secure:
                 elif len(self.auth_checkers) > 1:
                     sign_checker = self.sign_checkers[name]
                 else:
-                    raise Exception("No Sign Checker defined in Config")
+                    raise AuthError("No Sign Checker defined in Config")
                 # Assumes the request is available
                 token = get_token_header("signature", request)
                 body = request.get_data()
                 if not sign_checker.verify(token, body, json_dec, issuers):
-                    raise Exception("Wrong Signature", 401)
+                    raise AuthError("Wrong Signature", 401)
 
                 return f(*args, **kwargs)
 
@@ -166,18 +173,67 @@ class Secure:
 
         return inner_sign_check
 
-    def sign_make(self, body, audience, name=None):
+    def auth_or_sign_check(self, scope=None, name=None, json_dec=False, issuers=None):
+        def inner_auth_or_sign_check(f):
+            @wraps(f)
+            def decorated(*args, **kwargs):
+                auth = request.headers.get("Authorization", None)
+                if not auth:
+                    raise AuthError("No Authorization Header")
+
+                auth_parts = auth.split()
+
+                if auth_parts[0].lower() not in ["signature", "bearer"]:
+                    raise AuthError("Header should start Bearer or Signature")
+                elif len(auth_parts) == 1:
+                    raise AuthError("No Token attached")
+                elif len(auth_parts) > 2:
+                    raise AuthError("Too many parts in Token")
+                token = auth_parts[1]
+                auth_type = auth_parts[0].lower()
+                if auth_type == "signature":
+                    if len(self.sign_checkers) == 1:
+                        sign_checker = next(iter(self.sign_checkers.values()))
+                    elif len(self.auth_checkers) > 1:
+                        sign_checker = self.sign_checkers[name]
+                    else:
+                        raise AuthError("No Sign Checker defined in Config")
+                    # Assumes the request is available
+                    body = request.get_data()
+                    if not sign_checker.verify(token, body, json_dec, issuers):
+                        raise AuthError("Signature can't validate Body")
+
+                elif auth_type == "bearer":
+                    if len(self.auth_checkers) == 1:
+                        auth_checker = next(iter(self.auth_checkers.values()))
+                    elif len(self.auth_checkers) > 1:
+                        auth_checker = self.auth_checkers[name]
+                    else:
+                        raise AuthError("No Auth Checker defined in Config")
+                    auth_checker.decode(token, issuers)
+                    if not self.validate_scope(scope, token):
+                        raise AuthError("Invalid Scope")
+
+                return f(*args, **kwargs)
+
+            return decorated
+
+        return inner_auth_or_sign_check
+
+    def sign_make(self, body, audience, name=None, is_binary=False):
         if len(self.sign_makers) == 1:
             sign_maker = next(iter(self.sign_makers.values()))
         elif len(self.sign_makers) > 1:
             sign_maker = self.sign_makers[name]
         else:
-            raise Exception("No Sign Maker defined in Config")
-        return sign_maker.sign(body, audience)
+            raise AuthError("No Sign Maker defined in Config")
+        return sign_maker.sign(body, audience, is_binary)
 
-    def signed_headers(self, body, audience, name=None):
+    def signed_headers(self, body, audience, name=None, is_binary=False):
         headers = {
-            "Authorization": "Signature {}".format(self.sign_make(body, audience, name))
+            "Authorization": "Signature {}".format(
+                self.sign_make(body, audience, name, is_binary)
+            )
         }
         return headers
 
@@ -194,7 +250,7 @@ class KeySet:
                 url = issuer + ".well-known/jwks.json"
                 keys = requests.get(url=url).json()["keys"]
             except:
-                raise Exception(
+                raise AuthError(
                     "No keys provided nor found on {}.well-known/jwks.json".format(
                         issuer
                     )
@@ -230,28 +286,35 @@ class KeySet:
 
 class Authenticator:
     # Using Auth0
-    def __init__(self, domain, client_id, client_secret, audience):
+    def __init__(
+        self,
+        domain,
+        client_id,
+        client_secret,
+        audience,
+        grant_type="client_credentials",
+    ):
         self.domain = domain
         self.client_id = client_id
         self.client_secret = client_secret
         self.audience = audience
+        self.grant_type = grant_type
         self.getAccessToken()
         if self._access_token is None:
-            raise Exception("Request for access token failed")
+            raise AuthError("Request for access token failed")
 
-    def getAccessToken(self):
+    def getAccessToken(self, scope=""):
         body = {
             "client_id": self.client_id,
             "client_secret": self.client_secret,
             "audience": self.audience,
-            "grant_type": "client_credentials",
+            "grant_type": self.grant_type,
+            "scope": scope,
         }
         try:
-            response = requests.post(
-                url="https://{}/oauth/token".format(self.domain), json=body
-            )
+            response = requests.post(url=self.domain, json=body)
         except Exception as e:
-            raise Exception(e)
+            raise AuthError(e)
             return None
         else:
             json_response = DicPlus.json_loads(response.text)
@@ -261,9 +324,9 @@ class Authenticator:
             )
 
     @property
-    def access_token(self):
+    def access_token(self, scope=""):
         if time.time() > self._access_token_expiration:
-            self.getAccessToken()
+            self.getAccessToken(scope)
         return self._access_token
 
 
@@ -277,11 +340,11 @@ class Authorizer:
             jwks = self.keySet.get_keys(issuers)
             payload = jwt.decode(token, jwks, audience=self.audience)
         except jwt.ExpiredSignatureError:
-            raise Exception("Expired Signature", 401)
+            raise AuthError("Expired Signature", 401)
         except jwt.JWTClaimsError:
-            raise Exception("Please check audience and issuer", 401)
-        except Exception:
-            raise Exception("Invalid header", 401)
+            raise AuthError("Please check audience and issuer", 401)
+        except Exception as e:
+            raise AuthError("Invalid Token", e)
 
         return payload
 
@@ -304,11 +367,11 @@ class Signature_Verifyer:
         else:
             hashed_body = hash_digest(body)
         if verified_token.digest != hashed_body:
-            raise Exception("Digest Not Equal")
+            raise AuthError("Digest Not Equal")
         if verified_token.aud != self.audience:
-            raise Exception("Audience Not Equal")
+            raise AuthError("Audience Not Equal")
         if verified_token.exp < timegm(datetime.utcnow().utctimetuple()):
-            raise Exception("Token Expired")
+            raise AuthError("Token Expired")
         return True
 
 
@@ -321,10 +384,14 @@ class Signer:
         self.issuer = issuer
         self.private_key = private_key
 
-    def sign(self, body, audience, is_json=False):
+    def sign(self, body, audience, is_binary=False):
         jti = str(uuid.uuid4())
-        enc_body = json.dumps(body, separators=(",", ": ")).encode()
+        if is_binary:
+            enc_body = body
+        else:
+            enc_body = json.dumps(body, separators=(",", ": ")).encode()
         digest = hash_digest(enc_body)
+        print(digest)
         token_payload = {
             "digest": digest,
             "jti": jti,
